@@ -15,15 +15,15 @@ use constant IS_MOD_PERL_2       =>
     eval { require mod_perl } && $mod_perl::VERSION >= 1.99;
 
 use constant IS_MOD_PERL_2_BUILD => IS_MOD_PERL_2 &&
-    eval { require Apache::Build } && Apache::Build::IS_MOD_PERL_BUILD();
+    require Apache::Build && Apache::Build::IS_MOD_PERL_BUILD();
 
 use Symbol ();
 use File::Copy ();
 use File::Find qw(finddepth);
 use File::Basename qw(dirname);
 use File::Path ();
-use File::Spec::Functions qw(catfile abs2rel splitdir
-                             catdir file_name_is_absolute);
+use File::Spec::Functions qw(catfile abs2rel splitdir canonpath
+                             catdir file_name_is_absolute devnull);
 use Cwd qw(fastcwd);
 
 use Apache::TestConfigPerl ();
@@ -35,36 +35,38 @@ use Socket ();
 use vars qw(%Usage);
 
 %Usage = (
-   top_dir       => 'top-level directory (default is $PWD)',
-   t_dir         => 'the t/ test directory (default is $top_dir/t)',
-   t_conf        => 'the conf/ test directory (default is $t_dir/conf)',
-   t_logs        => 'the logs/ test directory (default is $t_dir/logs)',
-   t_conf_file   => 'test httpd.conf file (default is $t_conf/httpd.conf)',
-   src_dir       => 'source directory to look for mod_foos.so',
-   serverroot    => 'ServerRoot (default is $t_dir)',
-   documentroot  => 'DocumentRoot (default is $ServerRoot/htdocs',
-   port          => 'Port [port_number|select] (default ' . DEFAULT_PORT . ')',
-   servername    => 'ServerName (default is localhost)',
-   user          => 'User to run test server as (default is $USER)',
-   group         => 'Group to run test server as (default is $GROUP)',
-   bindir        => 'Apache bin/ dir (default is apxs -q BINDIR)',
-   sbindir       => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
-   httpd         => 'server to use for testing (default is $bindir/httpd)',
-   target        => 'name of server binary (default is apxs -q TARGET)',
-   apxs          => 'location of apxs (default is from Apache::BuildConfig)',
-   httpd_conf    => 'inherit config from this file (default is apxs derived)',
-   maxclients    => 'maximum number of concurrent clients (default is 1)',
-   perlpod       => 'location of perl pod documents (for testing downloads)',
-   proxyssl_url  => 'url for testing ProxyPass / https (default is localhost)',
-   sslca         => 'location of SSL CA (default is $t_conf/ssl/ca)',
-   sslcaorg      => 'SSL CA organization to use for tests (default is asf)',
+   top_dir         => 'top-level directory (default is $PWD)',
+   t_dir           => 'the t/ test directory (default is $top_dir/t)',
+   t_conf          => 'the conf/ test directory (default is $t_dir/conf)',
+   t_logs          => 'the logs/ test directory (default is $t_dir/logs)',
+   t_conf_file     => 'test httpd.conf file (default is $t_conf/httpd.conf)',
+   src_dir         => 'source directory to look for mod_foos.so',
+   serverroot      => 'ServerRoot (default is $t_dir)',
+   documentroot    => 'DocumentRoot (default is $ServerRoot/htdocs',
+   port            => 'Port [port_number|select] (default ' . DEFAULT_PORT . ')',
+   servername      => 'ServerName (default is localhost)',
+   user            => 'User to run test server as (default is $USER)',
+   group           => 'Group to run test server as (default is $GROUP)',
+   bindir          => 'Apache bin/ dir (default is apxs -q BINDIR)',
+   sbindir         => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
+   httpd           => 'server to use for testing (default is $bindir/httpd)',
+   target          => 'name of server binary (default is apxs -q TARGET)',
+   apxs            => 'location of apxs (default is from Apache::BuildConfig)',
+   startup_timeout => 'seconds to wait for the server to start (default is 60)',
+   httpd_conf      => 'inherit config from this file (default is apxs derived)',
+   maxclients      => 'maximum number of concurrent clients (default is 1)',
+   perlpod         => 'location of perl pod documents (for testing downloads)',
+   proxyssl_url    => 'url for testing ProxyPass / https (default is localhost)',
+   sslca           => 'location of SSL CA (default is $t_conf/ssl/ca)',
+   sslcaorg        => 'SSL CA organization to use for tests (default is asf)',
+   libmodperl      => 'path to mod_perl\'s .so (full or relative to LIBEXECDIR)',
    (map { $_ . '_module_name', "$_ module name"} qw(cgi ssl thread access auth)),
 );
 
 sub usage {
     for my $hash (\%Usage) {
         for (sort keys %$hash){
-            printf "   -%-16s %s\n", $_, $hash->{$_};
+            printf "  -%-18s %s\n", $_, $hash->{$_};
         }
     }
 }
@@ -82,7 +84,7 @@ sub filter_args {
     while (@filter) {
         my $key = shift @filter;
         # optinal - or -- prefix
-        if ($key =~ /^-?-?(.+)/ && exists $wanted_args->{$1}) {
+        if (defined $key && $key =~ /^-?-?(.+)/ && exists $wanted_args->{$1}) {
             if (@filter) {
                 $keep{$1} = shift @filter;
             }
@@ -214,7 +216,8 @@ sub new {
     $vars->{t_dir}        ||= catfile $vars->{top_dir}, 't';
     $vars->{serverroot}   ||= $vars->{t_dir};
     $vars->{documentroot} ||= catfile $vars->{serverroot}, 'htdocs';
-    $vars->{perlpod}      ||= $self->find_in_inc('pod');
+    $vars->{perlpod}      ||= $self->find_in_inc('pods') ||
+                              $self->find_in_inc('pod');
     $vars->{perl}         ||= $^X;
     $vars->{t_conf}       ||= catfile $vars->{serverroot}, 'conf';
     $vars->{sslca}        ||= catfile $vars->{t_conf}, 'ssl', 'ca';
@@ -282,6 +285,8 @@ sub configure_apxs {
 
     return unless $self->{APXS};
 
+    $self->{APXS} =~ s{/}{\\}g if WIN32;
+
     my $vars = $self->{vars};
 
     $vars->{bindir}   ||= $self->apxs('BINDIR', 1);
@@ -319,6 +324,7 @@ sub configure_httpd {
         #handle both $prefix/bin/httpd and $prefix/Apache.exe
         for (1,2) {
             pop @chunks;
+            last unless @chunks;
             $self->{httpd_basedir} = catfile @chunks;
             last if -d "$self->{httpd_basedir}/bin";
         }
@@ -367,25 +373,25 @@ sub configure_proxy {
 sub add_config {
     my $self = shift;
     my $where = shift;
-    my($directive, $arg, $hash) = @_;
+    my($directive, $arg, $data) = @_;
     my $args = "";
 
-    if ($hash) {
+    if ($data) {
         $args = "<$directive $arg>\n";
-        if (ref($hash)) {
-            while (my($k,$v) = each %$hash) {
-                if (ref($v) eq 'ARRAY') {
-                    for (@$v) {
-                        $args .= "    $k $_\n";
-                    }
-                }
-                else {
-                    $args .= "    $k $v\n";
-                }
+        if (ref($data) eq 'HASH') {
+            while (my($k,$v) = each %$data) {
+                $args .= "    $k $v\n";
+            }
+        }
+        elsif (ref($data) eq 'ARRAY') {
+            # balanced (key=>val) list
+            my $pairs = @$data / 2;
+            for my $i (0..($pairs-1)) {
+                $args .= sprintf "    %s %s\n", $data->[$i*2], $data->[$i*2+1];
             }
         }
         else {
-            $args .= "    $hash";
+            $args .= "    $data";
         }
         $args .= "</$directive>\n";
     }
@@ -626,6 +632,8 @@ sub hostport {
 sub find_apache_module {
     my($self, $module) = @_;
 
+    die "find_apache_module: module name argument is required" unless $module;
+
     my $vars = $self->{vars};
     my $sroot = $vars->{serverroot};
 
@@ -682,9 +690,11 @@ sub warn_style_sub_ref {
 }
 
 sub genwarning {
-    my($self, $filename) = @_;
+    my($self, $filename, $from_filename) = @_;
     return unless $filename;
-    my $warning = "WARNING: this file is generated, do not edit\n";
+    my $warning = "WARNING: this file is generated";
+    $warning .= " (from $from_filename)" if defined $from_filename;
+    $warning .= ", do not edit\n";
     $warning .= calls_trace();
     return $self->warn_style_sub_ref($filename)->($warning);
 }
@@ -723,32 +733,35 @@ sub clean_add_path {
 }
 
 sub genfile_trace {
-    my($self, $file) = @_;
+    my($self, $file, $from_file) = @_;
     my $name = abs2rel $file, $self->{vars}->{t_dir};
-    debug "generating $name";
+    my $msg = "generating $name";
+    $msg .= " from $from_file" if defined $from_file;
+    debug $msg;
 }
 
 sub genfile_warning {
-    my($self, $file, $fh) = @_;
+    my($self, $file, $from_file, $fh) = @_;
 
-    if (my $msg = $self->genwarning($file)) {
+    if (my $msg = $self->genwarning($file, $from_file)) {
         print $fh $msg, "\n";
     }
 }
 
+# $from_file == undef if there was no templates used
 sub genfile {
-    my($self, $file, $nowarning) = @_;
+    my($self, $file, $from_file, $nowarning) = @_;
 
     # create the parent dir if it doesn't exist yet
     my $dir = dirname $file;
     $self->makepath($dir);
 
-    $self->genfile_trace($file);
+    $self->genfile_trace($file, $from_file);
 
     my $fh = Symbol::gensym();
     open $fh, ">$file" or die "open $file: $!";
 
-    $self->genfile_warning($file, $fh) unless $nowarning;
+    $self->genfile_warning($file, $from_file, $fh) unless $nowarning;
 
     $self->clean_add_file($file);
 
@@ -759,7 +772,7 @@ sub genfile {
 sub writefile {
     my($self, $file, $content, $nowarning) = @_;
 
-    my $fh = $self->genfile($file, $nowarning);
+    my $fh = $self->genfile($file, undef, $nowarning);
 
     print $fh $content if $content;
 
@@ -767,14 +780,41 @@ sub writefile {
 }
 
 sub perlscript_header {
-    return <<'EOF';
+
+    require FindBin;
+
+    my @dirs = ();
+
+    # mp2 needs its modper-2.0/lib before blib was created
+    if (IS_MOD_PERL_2_BUILD || $ENV{APACHE_TEST_LIVE_DEV}) {
+        # the live 'lib/' dir of the distro
+        # (e.g. modperl-2.0/ModPerl-Registry/lib)
+        my $dir = canonpath catdir $FindBin::Bin, "lib";
+        push @dirs, $dir if -d $dir;
+
+        # the live dir of the top dir if any  (e.g. modperl-2.0/lib)
+        if (-e catfile($FindBin::Bin, "..", "Makefile.PL")) {
+            my $dir = canonpath catdir $FindBin::Bin, "..", "lib";
+            push @dirs, $dir if -d $dir;
+        }
+    }
+
+    for (qw(. ..)) {
+        my $dir = canonpath catdir $FindBin::Bin, $_ , "Apache-Test", "lib";
+        if (-d $dir) {
+            push @dirs, $dir;
+            last;
+        }
+    }
+
+    my $dirs = join("\n    ", '', @dirs) . "\n";;
+
+    return <<"EOF";
 
 use strict;
 use warnings FATAL => 'all';
 
-use FindBin;
-use lib map "$FindBin::Bin/$_",
-        qw(../Apache-Test/lib ../lib ../../lib);
+use lib qw($dirs);
 
 EOF
 }
@@ -783,12 +823,12 @@ EOF
 sub write_perlscript {
     my($self, $file, $content) = @_;
 
-    my $fh = $self->genfile($file, 1);
+    my $fh = $self->genfile($file, undef, 1);
 
     # shebang
     print $fh "#!$Config{perlpath}\n";
 
-    $self->genfile_warning($file, $fh);
+    $self->genfile_warning($file, undef, $fh);
 
     print $fh $content if $content;
 
@@ -902,10 +942,11 @@ sub servername_config {
 sub parse_vhost {
     my($self, $line) = @_;
 
-    my($indent, $module);
-    if ($line =~ /^(\s*)<VirtualHost\s+(?:_default_:)?(.*?)\s*>\s*$/) {
-        $indent = $1 || "";
-        $module = $2;
+    my($indent, $module, $namebased);
+    if ($line =~ /^(\s*)<VirtualHost\s+(?:_default_:|([^:]+):(?!:))?(.*?)\s*>\s*$/) {
+        $indent    = $1 || "";
+        $namebased = $2 || "";
+        $module    = $3;
     }
     else {
         return undef;
@@ -931,14 +972,23 @@ sub parse_vhost {
     }
 
     #allocate a port and configure this module into $self->{vhosts}
-    my $port = $self->new_vhost($module);
+    my $port = $self->new_vhost($module, $namebased);
 
     #extra config that should go *inside* the <VirtualHost ...>
-    my @in_config = $self->servername_config($vars->{servername},
+    my @in_config = $self->servername_config($namebased
+                                                 ? $namebased
+                                                 : $vars->{servername},
                                              $port);
 
-    #extra config that should go *outside* the <VirtualHost ...>
-    my @out_config = ([Listen => $port]);
+    my @out_config = ();
+    if ($self->{vhosts}->{$module}->{namebased} < 2) {
+        #extra config that should go *outside* the <VirtualHost ...>
+        @out_config = ([Listen => $port]);
+
+        if ($self->{vhosts}->{$module}->{namebased}) {
+            push @out_config => [NameVirtualHost => "*:$port"];
+        }
+    }
 
     #there are two ways of building a vhost
     #first is when we parse test .pm and .c files
@@ -964,7 +1014,8 @@ sub parse_vhost {
         #used when parsing *.conf.in files
         in_string     => $form_string->($double_indent, @in_config),
         out_string    => $form_string->($indent, @out_config),
-        line          => "$indent<VirtualHost _default_:$port>",
+        line          => "$indent<VirtualHost " . ($namebased ? '*' : '_default_') .
+                         ":$port>",
     };
 }
 
@@ -1019,6 +1070,14 @@ EOF
 sub generate_types_config {
     my $self = shift;
 
+    # handle the case when mod_mime is built as a shared object
+    # but wasn't included in the system-wide httpd.conf
+    my $mod_mime = $self->find_apache_module('mod_mime.so');
+    if ($mod_mime && -e $mod_mime) {
+        $self->preamble(IfModule => '!mod_mime.c',
+                        qq{LoadModule mime_module "$mod_mime"\n});
+    }
+
     unless ($self->{inherit_config}->{TypesConfig}) {
         my $types = catfile $self->{vars}->{t_conf}, 'mime.types';
         unless (-e $types) {
@@ -1096,7 +1155,7 @@ sub generate_extra_conf {
         my $in = Symbol::gensym();
         open($in, $file) or next;
 
-        my $out = $self->genfile($generated);
+        my $out = $self->genfile($generated, $file);
         $self->replace_vars($in, $out);
 
         close $in;
@@ -1186,6 +1245,11 @@ sub prepare_t_conf {
     $self->gendir($self->{vars}->{t_conf});
 }
 
+my %aliases = (
+    "perl-pod"     => "perlpod",
+    "binary-httpd" => "httpd",
+    "binary-perl"  => "perl",
+);
 sub generate_httpd_conf {
     my $self = shift;
     my $vars = $self->{vars};
@@ -1205,7 +1269,7 @@ sub generate_httpd_conf {
                 $entry = qq(Include "$file");
             }
             elsif ($file =~ /\.pl$/) {
-                $entry = qq(PerlRequire "$file");
+                $entry = qq(<IfModule mod_perl.c>\n    PerlRequire "$file"\n</IfModule>\n);
             }
             else {
                 next;
@@ -1250,6 +1314,24 @@ sub generate_httpd_conf {
 
     $self->replace_vars($in, $out);
 
+    # handle the case when mod_alias is built as a shared object
+    # but wasn't included in the system-wide httpd.conf
+    my $mod_alias = $self->find_apache_module('mod_alias.so');
+    if ($mod_alias && -e $mod_alias) {
+        print $out <<EOF;
+<IfModule !mod_alias.c>
+    LoadModule alias_module "$mod_alias"
+</IfModule>
+EOF
+    }
+
+    print $out "<IfModule mod_alias.c>\n";
+    for (keys %aliases) {
+        next unless $vars->{$aliases{$_}};
+        print $out "    Alias /getfiles-$_ $vars->{$aliases{$_}}\n";
+    }
+    print $out "</IfModule>\n";
+
     print $out "\n";
 
     $self->postamble_run($out);
@@ -1339,7 +1421,8 @@ sub which {
 sub apxs {
     my($self, $q, $ok_fail) = @_;
     return unless $self->{APXS};
-    my $val = qx($self->{APXS} -q $q 2>/dev/null);
+    my $devnull = devnull();
+    my $val = qx($self->{APXS} -q $q 2>$devnull);
     chomp $val if defined $val; # apxs post-2.0.40 adds a new line
     unless ($val) {
         if ($ok_fail) {
@@ -1367,11 +1450,21 @@ sub add_inc {
     my $self = shift;
     return if $ENV{MOD_PERL}; #already setup by mod_perl
     require lib;
-    # make sure that the Apache-Test dev libs will be first in @INC,
-    # followed by modperl's lib, followed by blib and finally core
-    # Perl libs.
-    lib::->import(map "$self->{vars}->{top_dir}/$_",
-                  qw(Apache-Test/lib lib blib/lib blib/arch));
+    # make sure that Apache-Test/lib will be first in @INC,
+    # followed by modperl-2.0/lib (or some other project's lib/),
+    # followed by blib/ and finally system-wide libs.
+    my $top_dir = $self->{vars}->{top_dir};
+    my @dirs = map { catdir $top_dir, "blib", $_ } qw(lib arch);
+
+    my $apache_test_dir = catdir $top_dir, "Apache-Test";
+    unshift @dirs, $apache_test_dir if -d $apache_test_dir;
+
+    if ($ENV{APACHE_TEST_LIVE_DEV}) {
+        my $lib_dir = catdir $top_dir, "lib";
+        push @dirs, $lib_dir if -d $lib_dir;
+    }
+
+    lib::->import(@dirs);
     #print join "\n", "add_inc", @INC, "";
 }
 
@@ -1502,6 +1595,11 @@ file. After the warning a perl trace of calls to this this function is
 appended. This trace is useful for finding what code has created the
 file.
 
+  my $warn = $cfg->genwarning($filename, $from_filename)
+
+If C<$from_filename> is specified it'll be used in the warning to tell
+which file it was generated from.
+
 genwarning() automatically recognizes the comment type based on the
 file extension. If the extension is not recognized, the default C<#>
 style is used.
@@ -1516,15 +1614,29 @@ styles.
 genfile() creates a new file C<$file> for writing and returns a file
 handle.
 
-A comment with a warning and calls trace is added to the top of this
-file. See genwarning() for more info about this comment.
-
 If parent directories of C<$file> don't exist they will be
 automagically created.
 
 The file C<$file> and any created parent directories (if found empty)
 will be automatically removed on cleanup.
 
+A comment with a warning and calls trace is added to the top of this
+file. See genwarning() for more info about this comment.
+
+  my $fh = $cfg->genfile($file, $from_file);
+
+If C<$from_filename> is specified it'll be used in the warning to tell
+which file it was generated from.
+
+  my $fh = $cfg->genfile($file, $from_file, $nowarning);
+
+If C<$nowarning> is true, the warning won't be added. If using this
+optional argument and there is no C<$from_file> you must pass undef as
+in:
+
+  my $fh = $cfg->genfile($file, undef, $nowarning);
+
+
 =item writefile()
 
   $cfg->writefile($file, $content, [$nowarning]);
@@ -1563,6 +1675,29 @@ automatically removed on cleanup if found empty.
 
 =back
 
+=head1 Environment Variables
+
+The following environment variables affect the configuration and the
+run-time of the C<Apache::Test> framework:
+
+=head2 APACHE_TEST_COLOR
+
+To aid visual control over the configuration process and the run-time
+phase, C<Apache::Test> uses coloured fonts when the environment
+variable C<APACHE_TEST_COLOR> is set to a true value.
+
+=head2 APACHE_TEST_LIVE_DEV
+
+When using C<Apache::Test> during the project development phase, it's
+often convenient to have the I<project/lib> (live) directory appearing
+first in C<@INC> so any changes to the Perl modules, residing in it,
+immediately affect the server, without a need to rerun C<make> to
+update I<blib/lib>. When the environment variable
+C<APACHE_TEST_LIVE_DEV> is set to a true value during the
+configuration phase (C<t/TEST -config>, C<Apache::Test> will
+automatically unshift the I<project/lib> directory into C<@INC>, via
+the autogenerated I<t/conf/modperl_inc.pl> file.
+
 =head1 AUTHOR
 
 =head1 SEE ALSO
@@ -1581,7 +1716,10 @@ DocumentRoot "@DocumentRoot@"
 PidFile     @t_logs@/httpd.pid
 ErrorLog    @t_logs@/error_log
 LogLevel    debug
-TransferLog @t_logs@/access_log
+
+<IfModule mod_log_config.c>
+    TransferLog @t_logs@/access_log
+</IfModule>
 
 ServerAdmin @ServerAdmin@
 
@@ -1619,6 +1757,12 @@ HostnameLookups Off
     MaxRequestsPerChild  0
 </IfModule>
 
+<IfDefine APACHE1>
+    StartServers         1
+    MaxClients           @MaxClients@
+    MaxRequestsPerChild  0
+</IfDefine>
+
 <IfModule mpm_winnt.c>
     ThreadsPerChild      20
     MaxRequestsPerChild  0
@@ -1632,9 +1776,3 @@ HostnameLookups Off
     SetHandler server-status
 </Location>
 
-#so we can test downloading some files of various size
-Alias /getfiles-perl-pod          @PerlPod@
-
-#and some big ones
-Alias /getfiles-binary-httpd      @httpd@
-Alias /getfiles-binary-perl       @perl@
