@@ -17,6 +17,7 @@ our $VERSION = '1.99';
 
 use Apache::Response ();
 use Apache::RequestRec ();
+use Apache::RequestUtil ();
 use Apache::RequestIO ();
 use Apache::Log ();
 use Apache::Access ();
@@ -524,7 +525,7 @@ sub read_script {
     my $self = shift;
 
     $self->debug("reading $self->{FILENAME}") if DEBUG & D_NOISE;
-    $self->{CODE} = $self->{REQ}->my_slurp_filename;
+    $self->{CODE} = $self->{REQ}->slurp_filename(0); # untainted
 }
 
 #########################################################################
@@ -639,7 +640,6 @@ sub compile {
 
     ModPerl::Global::special_list_clear(END => $self->{PACKAGE});
 
-    ModPerl::Util::untaint($$eval);
     {
         # let the code define its own warn and strict level 
         no strict;
@@ -747,23 +747,14 @@ sub uncache_myself {
 }
 
 
-# XXX: these should go away when finfo() and slurp_filename() are
-# ported to 2.0 (don't want to depend on compat.pm)
+# XXX: should go away when finfo() is ported to 2.0 (don't want to
+# depend on compat.pm)
 sub Apache::RequestRec::my_finfo {
     my $r = shift;
     stat $r->filename;
     \*_;
 }
 
-sub Apache::RequestRec::my_slurp_filename {
-    my $r = shift;
-    open my $fh, $r->filename;
-    local $/;
-    my $data = <$fh>;
-    close $fh;
-    return \$data;
-}
-
 
 1;
 __END__
