@@ -6,7 +6,7 @@ use warnings;
 
 use Config;
 use Cwd ();
-use File::Spec::Functions qw(catfile catdir canonpath rel2abs);
+use File::Spec::Functions qw(catfile catdir canonpath rel2abs devnull);
 use File::Basename;
 use ExtUtils::Embed ();
 
@@ -127,7 +127,8 @@ sub apxs {
         return defined $val ? ($val ? "$prefix/$val" : $prefix) : "";
     }
 
-    my $val = qx($apxs @_ 2>/dev/null);
+    my $devnull = devnull();
+    my $val = qx($apxs @_ 2>$devnull);
     chomp $val if defined $val; # apxs post-2.0.40 adds a new line
 
     unless ($val) {
@@ -252,9 +253,13 @@ sub ldopts {
     $ldopts;
 }
 
-my $Wall = 
+my $Wall =
   "-Wall -Wmissing-prototypes -Wstrict-prototypes -Wmissing-declarations";
 
+# perl v5.6.1 and earlier produces lots of warnings, so we can't use
+# -Werror with those versions.
+$Wall .= " -Werror" if $] >= 5.006002;
+
 sub ap_ccopts {
     my($self) = @_;
     my $ccopts = "-DMOD_PERL";
@@ -1515,6 +1520,24 @@ BEGIN {
     if (@dirs) {
         unshift @INC, @dirs;
     }
+
+    # now re-org the libs to have first devel libs, then blib libs,
+    # and only then perl core libs
+    use File::Basename qw(dirname);
+    my $project_root = $blib ? dirname(dirname($blib)) : '';
+    if ($project_root) {
+        my (@a, @b, @c);
+        for (@INC) {
+            if (m|^\Q$project_root\E|) {
+                m|blib| ? push @b, $_ : push @a, $_;
+            }
+            else {
+                push @c, $_;
+            }
+        }
+        @INC = (@a, @b, @c);
+    }
+
 }
 EOF
 
