@@ -9,71 +9,6 @@ static int MP_init_status = 0;
 #define MP_IS_STARTING    (MP_init_status == 1 ? 1 : 0)
 #define MP_IS_RUNNING     (MP_init_status == 2 ? 1 : 0)
 
-#if !(PERL_REVISION == 5 && ( PERL_VERSION < 8 || \
-    (PERL_VERSION == 8 && PERL_SUBVERSION == 0))) && \
-    (defined(USE_HASH_SEED) || defined(USE_HASH_SEED_EXPLICIT))
-#define MP_NEED_HASH_SEED_FIXUP
-#endif
-
-
-#ifdef MP_NEED_HASH_SEED_FIXUP
-static UV   MP_init_hash_seed = 0;
-static bool MP_init_hash_seed_set = FALSE;
-#endif
-
-/* see modperl_hash_seed_set() */
-static void modperl_hash_seed_init(apr_pool_t *p) 
-{
-#ifdef MP_NEED_HASH_SEED_FIXUP
-    char *s;
-    /* check if there is a specific hash seed passed via the env var
-     * and if that's the case -- use it */
-    apr_status_t rv = apr_env_get(&s, "PERL_HASH_SEED", p);
-    if (rv == APR_SUCCESS) {
-        if (s) {
-            while (isSPACE(*s)) s++;
-        }
-        if (s && isDIGIT(*s)) {
-            MP_init_hash_seed = (UV)Atol(s); // XXX: Atoul()?
-            MP_init_hash_seed_set = TRUE;
-        }
-    }
-    
-    /* calculate our own random hash seed */
-    if (!MP_init_hash_seed_set) {
-        apr_uuid_t *uuid = (apr_uuid_t *)apr_palloc(p, sizeof(apr_uuid_t));
-        char buf[APR_UUID_FORMATTED_LENGTH + 1];
-        int i;
-
-        apr_initialize();
-        apr_uuid_get(uuid);
-        apr_uuid_format(buf, uuid);
-        /* fprintf(stderr, "UUID: %s\n", buf); */
-
-        /* XXX: need a better alg to convert uuid string into a seed */
-        for (i=0; buf[i]; i++){
-            MP_init_hash_seed += (UV)(i+1)*(buf[i]+MP_init_hash_seed);
-        }
-
-        MP_init_hash_seed_set = TRUE;
-    }
-#endif
-}
-
-/* before 5.8.1, perl was using HASH_SEED=0, starting from 5.8.1
- * it randomizes if perl was compiled with ccflags -DUSE_HASH_SEED
- * or -DUSE_HASH_SEED_EXPLICIT, in which case we need to tell perl
- * to use the same seed everywhere */
-static void modperl_hash_seed_set(pTHX) 
-{
-#ifdef MP_NEED_HASH_SEED_FIXUP
-    if (MP_init_hash_seed_set) {
-        PL_hash_seed_set = TRUE;
-        PL_hash_seed = MP_init_hash_seed;
-    }
-#endif
-}
-
 #ifndef USE_ITHREADS
 static apr_status_t modperl_shutdown(void *data)
 {
@@ -178,21 +113,23 @@ apr_pool_t *modperl_server_pool(void)
 static void set_taint_var(PerlInterpreter *perl)
 {
     dTHXa(perl);
-    GV *gv;
 
 /* 5.7.3+ has a built-in special ${^TAINT}, backport it to 5.6.0+ */
 #if PERL_REVISION == 5 && \
     (PERL_VERSION == 6 || (PERL_VERSION == 7 && PERL_SUBVERSION < 3))
-
-    gv = gv_fetchpv("\024AINT", GV_ADDMULTI, SVt_IV);
-    sv_setiv(GvSV(gv), PL_tainting);
-    SvREADONLY_on(GvSV(gv));
+    {
+        GV *gv = gv_fetchpv("\024AINT", GV_ADDMULTI, SVt_IV);
+        sv_setiv(GvSV(gv), PL_tainting);
+        SvREADONLY_on(GvSV(gv));
+    }
 #endif /* perl v < 5.7.3 */
 
 #ifdef MP_COMPAT_1X
-    gv = gv_fetchpv("Apache::__T", GV_ADDMULTI, SVt_PV);
-    sv_setiv(GvSV(gv), PL_tainting);
-    SvREADONLY_on(GvSV(gv));
+    {
+        GV *gv = gv_fetchpv("Apache::__T", GV_ADDMULTI, SVt_PV);
+        sv_setiv(GvSV(gv), PL_tainting);
+        SvREADONLY_on(GvSV(gv));
+    }
 #endif /* MP_COMPAT_1X */
     
 }
@@ -210,6 +147,36 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     modperl_cleanup_data_t *cdata;
 #endif
 
+    /* ensure that we start the base server's perl, before vhost's
+     * one, if modperl_startup was called by vhost before the former
+     * was started */
+    if (MP_init_status != 2) {
+        server_rec *base_server = modperl_global_get_server_rec();
+        PerlInterpreter *base_perl;
+        
+        MP_init_status = 2; /* calls itself, so set the flag early */
+        base_perl = modperl_startup(base_server, p);
+        
+        if (base_server == s ) {
+            return base_perl;
+        }
+    }
+
+#ifdef MP_TRACE
+    {
+        server_rec *base_server = modperl_global_get_server_rec();
+        const char *desc = modperl_server_desc(s, p);
+        if (base_server == s) {        
+            MP_TRACE_i(MP_FUNC,
+                       "starting the parent perl for the base server\n", desc);
+        }
+        else {
+            MP_TRACE_i(MP_FUNC,
+                       "starting the parent perl for vhost %s\n", desc);
+        }
+    }
+#endif
+
 #ifdef MP_USE_GTOP
     MP_TRACE_m_do(
         modperl_gtop_do_proc_mem_before(MP_FUNC, "perl_parse");
@@ -260,6 +227,9 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
 #ifdef USE_ITHREADS
     /* base server / virtual host w/ +Parent gets its own mip */
     modperl_interp_init(s, p, perl);
+
+    /* mark the parent perl to be destroyed */
+    MpInterpBASE_On(scfg->mip->parent);
 #endif
 
     PL_endav = endav;
@@ -275,7 +245,7 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     );
 #endif
 
-#ifdef MP_COMPAT_1X    
+#ifdef MP_COMPAT_1X
     av_push(GvAV(PL_incgv),
             newSVpv(ap_server_root_relative(p, ""), 0));
     av_push(GvAV(PL_incgv),
@@ -308,12 +278,20 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
     PerlInterpreter *perl;
     const char *vhost = modperl_server_desc(s, p);
 
+    if (!scfg) {
+        MP_TRACE_i(MP_FUNC, "server %s has no mod_perl config\n", vhost);
+        return OK;
+    }
+
     if (base_server == NULL) {
         base_server = modperl_global_get_server_rec();
     }
 
+    MP_TRACE_i(MP_FUNC, "Init vhost %s: s=0x%lx, base_s=0x%lx\n",
+               vhost, s, base_server);
+    
     if (base_server == s) {
-        MP_TRACE_i(MP_FUNC, "skipping vhost init for base server %s\n",
+        MP_TRACE_i(MP_FUNC, "base server is not vhost, skipping %s\n",
                    vhost);
         return OK;
     }
@@ -326,11 +304,6 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
     perl = base_perl = base_scfg->perl;
 #endif /* USE_ITHREADS */
 
-    if (!scfg) {
-        MP_TRACE_i(MP_FUNC, "server %s has no mod_perl config\n", vhost);
-        return OK;
-    }
-
 #ifdef USE_ITHREADS
 
     if (scfg->mip) {
@@ -375,11 +348,6 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
     }
 
 #ifdef USE_ITHREADS
-    /* if we allocated a parent perl, mark it to be destroyed */
-    if (MpSrvPARENT(scfg)) {
-        MpInterpBASE_On(scfg->mip->parent);
-    }
-    
     if (!scfg->mip) {
         /* since mips are created after merge_server_configs()
          * need to point to the base mip here if this vhost
@@ -420,18 +388,18 @@ void modperl_init(server_rec *base_server, apr_pool_t *p)
 
     base_perl = modperl_startup(base_server, p);
 
-    MP_init_status = 2; /* only now mp has really started */
-
-#ifdef USE_ITHREADS
-    /* a parent perl was allocated, mark it to be destroyed */
-    MpInterpBASE_On(base_scfg->mip->parent);
-#endif
-    
     for (s=base_server->next; s; s=s->next) {
         if (modperl_init_vhost(s, p, base_server) != OK) {
             exit(1); /*XXX*/
         }
     }
+
+#ifdef USE_ITHREADS
+    /* after other parent perls were started in vhosts, make sure that
+     * the context is set to the base_perl */
+    PERL_SET_CONTEXT(base_perl);
+#endif
+
 }
 
 #ifdef USE_ITHREADS
@@ -486,6 +454,8 @@ void modperl_init_globals(server_rec *s, apr_pool_t *pconf)
     int threaded_mpm;
     ap_mpm_query(AP_MPMQ_IS_THREADED, &threaded_mpm);
 
+    MP_TRACE_g(MP_FUNC, "mod_perl globals are configured\n");
+    
     modperl_global_init_pconf(pconf, pconf);
     modperl_global_init_threaded_mpm(pconf, threaded_mpm);
     modperl_global_init_server_rec(pconf, s);
