@@ -118,7 +118,7 @@ modperl_config_srv_t *modperl_config_srv_new(apr_pool_t *p)
     
     modperl_config_srv_argv_push((char *)ap_server_argv0);
 
-    MP_TRACE_d(MP_FUNC, "0x%lx\n", (unsigned long)scfg);
+    MP_TRACE_d(MP_FUNC, "new scfg: 0x%lx\n", (unsigned long)scfg);
 
     return scfg;
 }
@@ -134,7 +134,7 @@ modperl_config_dir_t *modperl_config_dir_new(apr_pool_t *p)
 
     dcfg->SetEnv = apr_table_make(p, 2);
     
-    MP_TRACE_d(MP_FUNC, "0x%lx\n", (unsigned long)dcfg);
+    MP_TRACE_d(MP_FUNC, "new dcfg: 0x%lx\n", (unsigned long)dcfg);
 
     return dcfg;
 }
@@ -144,9 +144,9 @@ static void dump_argv(modperl_config_srv_t *scfg)
 {
     int i;
     char **argv = (char **)scfg->argv->elts;
-    modperl_trace(NULL, "modperl_config_srv_argv_init =>\n");
+    modperl_trace(NULL, "modperl_config_srv_argv_init =>");
     for (i=0; i<scfg->argv->nelts; i++) {
-        modperl_trace(NULL, "   %d = %s\n", i, argv[i]);
+        modperl_trace(NULL, "   %d = %s", i, argv[i]);
     }
 }
 #endif
@@ -170,20 +170,23 @@ void *modperl_config_srv_create(apr_pool_t *p, server_rec *s)
 
     if (!s->is_virtual) {
 
+        /* give a chance to MOD_PERL_TRACE env var to set
+         * PerlTrace. This place is the earliest point in mod_perl
+         * configuration parsing, when we have the server object
+         */
+        modperl_trace_level_set(s, NULL);
+
         /* Must store the global server record as early as possible,
          * because if mod_perl happens to be started from within a
          * vhost (e.g., PerlLoadModule) the base server record won't
          * be available to vhost and things will blow up
          */
         modperl_init_globals(s, p);
-
-        /* give a chance to MOD_PERL_TRACE env var to set
-         * PerlTrace. This place is the earliest point in mod_perl
-         * configuration parsing, when we have the server object
-         */
-        modperl_trace_level_set(s, NULL);
     }
     
+    MP_TRACE_d(MP_FUNC, "p=0x%lx, s=0x%lx, virtual=%d\n",
+               p, s, s->is_virtual);
+
 #ifdef USE_ITHREADS
 
     scfg->interp_pool_cfg = 
