@@ -20,7 +20,8 @@ modperl_handler_t *modperl_handler_new(apr_pool_t *p, const char *name)
     }
 
     handler->name = name;
-    MP_TRACE_h(MP_FUNC, "new handler %s\n", handler->name);
+    MP_TRACE_h(MP_FUNC, "[%s] new handler %s\n",
+               modperl_pid_tid(p), handler->name);
 
     return handler;
 }
@@ -33,15 +34,12 @@ int modperl_handler_resolve(pTHX_ modperl_handler_t **handp,
 
 #ifdef USE_ITHREADS
     if (p && !MpHandlerPARSED(handler) && !MpHandlerDYNAMIC(handler)) {
-        MP_dSCFG(s);
-        if (scfg->threaded_mpm) {
-            /*
-             * cannot update the handler structure at request time without
-             * locking, so just copy it
-             */
-            handler = *handp = modperl_handler_dup(p, handler);
-            duped = 1;
-        }
+        /*
+         * cannot update the handler structure at request time without
+         * locking, so just copy it
+         */
+        handler = *handp = modperl_handler_dup(p, handler);
+        duped = 1;
     }
 #endif
 
@@ -52,8 +50,10 @@ int modperl_handler_resolve(pTHX_ modperl_handler_t **handp,
         MpHandlerAUTOLOAD_On(handler);
 
         MP_TRACE_h(MP_FUNC,
-                   "handler %s was not compiled at startup, "
+                   "[%s %s] handler %s was not compiled at startup, "
                    "attempting to resolve using %s pool 0x%lx\n",
+                   modperl_pid_tid(p),
+                   modperl_server_desc(s, p),
                    handler->name,
                    duped ? "current" : "server conf",
                    (unsigned long)rp);
@@ -222,8 +222,8 @@ MpAV **modperl_handler_lookup_handlers(modperl_config_dir_t *dcfg,
         break;
       case MP_HANDLER_TYPE_PRE_CONNECTION:
         avp = &scfg->handlers_pre_connection[idx];
-        check_modify(connection);
-        set_desc(connection);
+        check_modify(pre_connection);
+        set_desc(pre_connection);
         break;
       case MP_HANDLER_TYPE_CONNECTION:
         avp = &scfg->handlers_connection[idx];
