@@ -92,21 +92,23 @@ MP_INLINE GV *modperl_io_perlio_override_stdin(pTHX_ request_rec *r)
 {
     dHANDLE("STDIN");
     int status;
-    GV *handle_save = gv_fetchpv("STDIN_SAVED", TRUE, SVt_PVIO);
+    GV *handle_save = gv_fetchpv(Perl_form(aTHX_ "Apache::RequestIO::_GEN_%ld",
+                                           (long)PL_gensym++),
+                                 TRUE, SVt_PVIO);
     SV *sv = sv_newmortal();
 
     sv_setref_pv(sv, "Apache::RequestRec", (void*)r);
     MP_TRACE_o(MP_FUNC, "start");
 
-    /* open my $oldout, ">&STDIN" or die "Can't dup STDIN: $!"; */
-    status = Perl_do_open(aTHX_ handle_save, ">&STDIN", 8, FALSE, O_RDONLY,
+    /* open my $oldout, "<&STDIN" or die "Can't dup STDIN: $!"; */
+    status = Perl_do_open(aTHX_ handle_save, "<&STDIN", 7, FALSE, O_RDONLY,
                           0, Nullfp);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to dup STDIN: %_", get_sv("!", TRUE));
     }
 
     /* similar to PerlIO::scalar, the PerlIO::Apache layer doesn't
-     * have file descriptors, so STDOUT must be closed before it can
+     * have file descriptors, so STDIN must be closed before it can
      * be reopened */
     Perl_do_close(aTHX_ handle, TRUE); 
     status = Perl_do_open9(aTHX_ handle, "<:Apache", 8, FALSE, O_RDONLY,
@@ -125,7 +127,9 @@ MP_INLINE GV *modperl_io_perlio_override_stdout(pTHX_ request_rec *r)
 {
     dHANDLE("STDOUT");
     int status;
-    GV *handle_save = gv_fetchpv("STDOUT_SAVED", TRUE, SVt_PVIO);
+    GV *handle_save = gv_fetchpv(Perl_form(aTHX_ "Apache::RequestIO::_GEN_%ld",
+                                           (long)PL_gensym++),
+                                 TRUE, SVt_PVIO);
     SV *sv = sv_newmortal();
 
     MP_TRACE_o(MP_FUNC, "start");
@@ -133,7 +137,7 @@ MP_INLINE GV *modperl_io_perlio_override_stdout(pTHX_ request_rec *r)
     sv_setref_pv(sv, "Apache::RequestRec", (void*)r);
 
     /* open my $oldout, ">&STDOUT" or die "Can't dup STDOUT: $!"; */
-    status = Perl_do_open(aTHX_ handle_save, ">&STDOUT", 8, FALSE, O_RDONLY,
+    status = Perl_do_open(aTHX_ handle_save, ">&STDOUT", 8, FALSE, O_WRONLY,
                           0, Nullfp);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to dup STDOUT: %_", get_sv("!", TRUE));
@@ -143,7 +147,7 @@ MP_INLINE GV *modperl_io_perlio_override_stdout(pTHX_ request_rec *r)
      * have file descriptors, so STDOUT must be closed before it can
      * be reopened */
     Perl_do_close(aTHX_ handle, TRUE); 
-    status = Perl_do_open9(aTHX_ handle, ">:Apache", 8, FALSE, O_RDONLY,
+    status = Perl_do_open9(aTHX_ handle, ">:Apache", 8, FALSE, O_WRONLY,
                            0, Nullfp, sv, 1);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to open STDOUT: %_", get_sv("!", TRUE));
@@ -166,15 +170,16 @@ MP_INLINE void modperl_io_perlio_restore_stdin(pTHX_ GV *handle)
 
     /* Perl_do_close(aTHX_ handle_orig, FALSE); */
 
-    /* open STDIN, ">&STDIN_SAVED" or die "Can't dup STDIN_SAVED: $!"; */
+    /* open STDIN, "<&STDIN_SAVED" or die "Can't dup STDIN_SAVED: $!"; */
     status = Perl_do_open9(aTHX_ handle_orig, "<&", 2, FALSE, O_RDONLY,
                            0, Nullfp, (SV*)handle, 1);
+    Perl_do_close(aTHX_ handle, FALSE);
+    (void)hv_delete(gv_stashpv("Apache::RequestIO", TRUE), 
+		    GvNAME(handle), GvNAMELEN(handle), G_DISCARD);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to restore STDIN: %_", get_sv("!", TRUE));
     }
 
-    IoFLUSH_off(handle); /* STDIN's $|=0 */
-
     MP_TRACE_o(MP_FUNC, "end\n");
 }
 
@@ -188,8 +193,11 @@ MP_INLINE void modperl_io_perlio_restore_stdout(pTHX_ GV *handle)
     /* Perl_do_close(aTHX_ handle_orig, FALSE); */
 
     /* open STDOUT, ">&STDOUT_SAVED" or die "Can't dup STDOUT_SAVED: $!"; */
-    status = Perl_do_open9(aTHX_ handle_orig, ">&", 2, FALSE, O_RDONLY,
+    status = Perl_do_open9(aTHX_ handle_orig, ">&", 2, FALSE, O_WRONLY,
                            0, Nullfp, (SV*)handle, 1);
+    Perl_do_close(aTHX_ handle, FALSE);
+    (void)hv_delete(gv_stashpv("Apache::RequestIO", TRUE), 
+		    GvNAME(handle), GvNAMELEN(handle), G_DISCARD);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to restore STDOUT: %_", get_sv("!", TRUE));
     }
