@@ -242,4 +242,16 @@ typedef struct {
     char *path_info;
 } modperl_uri_t;
 
+typedef struct {
+    module *modp;
+    const char *cmd_data;
+    const char *func_name;
+} modperl_module_cmd_data_t;
+
+typedef enum {
+    MP_HOOK_RUN_ALL,
+    MP_HOOK_RUN_FIRST,
+    MP_HOOK_VOID
+} modperl_hook_run_mode_e;
+
 #endif /* MODPERL_TYPES_H */
