@@ -263,7 +263,7 @@ int modperl_errsv(pTHX_ int status, request_rec *r, server_rec *s)
         if (SvMAGICAL(sv) && (SvCUR(sv) > 4) &&
             strnEQ(SvPVX(sv), " at ", 4))
         {
-            /* Apache::exit was called */
+            /* ModPerl::Util::exit was called */
             return DECLINED;
         }
 #if 0
@@ -473,6 +473,12 @@ HE *modperl_perl_hv_fetch_he(pTHX_ HV *hv,
         return 0;
     }
 
+#ifdef HvREHASH
+    if (HvREHASH(hv)) {
+	PERL_HASH_INTERNAL(hash, key, klen);
+    }
+    else
+#endif
     if (!hash) {
 	PERL_HASH(hash, key, klen);
     }
