@@ -60,11 +60,11 @@
 /* check whether the response phase has been initialized already */
 #define MP_CHECK_WBUCKET_INIT(func) \
     if (!rcfg->wbucket) { \
-        Perl_croak(aTHX_ func " can't be called before the response phase"); \
+        Perl_croak(aTHX_ "%s: " func " can't be called before the response phase", MP_FUNC); \
     }
 
 /* turn off cgi header parsing. in case we are already inside
- *     modperl_callback_per_dir(MP_RESPONSE_HANDLER, r); 
+ *     modperl_callback_per_dir(MP_RESPONSE_HANDLER, r, MP_HOOK_RUN_FIRST); 
  * but haven't sent any data yet, it's too late to change
  * MpReqPARSE_HEADERS, so change the wbucket's private flag directly
  */
@@ -97,6 +97,8 @@ int modperl_require_file(pTHX_ const char *pv, int logfailure);
 
 char *modperl_server_desc(server_rec *s, apr_pool_t *p);
 
+MP_INLINE char *modperl_pid_tid(apr_pool_t *p);
+
 void modperl_xs_dl_handles_clear(pTHX);
 
 void **modperl_xs_dl_handles_get(pTHX);
@@ -141,8 +143,20 @@ SV *modperl_table_get_set(pTHX_ apr_table_t *table, char *key,
 
 MP_INLINE int modperl_perl_module_loaded(pTHX_ const char *name);
 
+/**
+ * slurp the contents of r->filename and return them as a scalar
+ * @param r       request record
+ * @param tainted whether the SV should be marked tainted or not
+ * @return a PV scalar with the contents of the file
+ */
+SV *modperl_slurp_filename(pTHX_ request_rec *r, int tainted);
+
 SV *modperl_perl_gensym(pTHX_ char *pack);
 
 void modperl_clear_symtab(pTHX_ HV *symtab);
 
+#ifdef MP_TRACE
+void modperl_apr_table_dump(pTHX_ apr_table_t *table, char *name);
+#endif
+
 #endif /* MODPERL_UTIL_H */
