@@ -6,6 +6,9 @@ package TestFilter::both_str_req_add;
 use strict;
 use warnings FATAL => 'all';
 
+use Apache::RequestRec ();
+use Apache::RequestIO ();
+
 use Apache::Filter ();
 
 use Apache::Const -compile => qw(OK M_POST);
