@@ -6,9 +6,8 @@ static MP_INLINE int mpxs_ap_run_sub_req(pTHX_ request_rec *r)
      */
 
     if (r->main) {
-        modperl_config_req_t *rcfg =
-            modperl_config_req_get(r->main);
-        modperl_wbucket_flush(rcfg->wbucket);
+        modperl_config_req_t *rcfg = modperl_config_req_get(r->main);
+        MP_FAILURE_CROAK(modperl_wbucket_flush(rcfg->wbucket, FALSE));
     }
 
     return ap_run_sub_req(r);
